#!/usr/bin/python3
"""
// © University of Southampton IT Innovation Centre, 2018
//
// Copyright in this software belongs to University of Southampton
// IT Innovation Centre of Gamma House, Enterprise Road,
// Chilworth Science Park, Southampton, SO16 7NS, UK.
//
// This software may not be used, sold, licensed, transferred, copied
// or reproduced in whole or in part in any manner or form or in or
// on any media by any person other than in accordance with the terms
// of the Licence Agreement supplied with the software, or otherwise
// without the prior written consent of the copyright owners.
//
// This software is distributed WITHOUT ANY WARRANTY, without even the
// implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
// PURPOSE, except where stated in the Licence Agreement supplied with
// the software.
//
//      Created By :            Nikolay Stanchev
//      Created Date :          25-06-2019
//      Created for Project :   FLAME
"""


# Python standard libs
from unittest.mock import patch, Mock

# PIP installed libs
import pytest
from pyramid import testing

# CLMC-service imports
from clmcservice.managementapi.views import DataManagementAPI


class TestDataManagementAPI(object):
    """
    A pytest-implementation test for the Data Management API endpoints.
    """

    @pytest.fixture(autouse=True)
    def print_fixture(self):
        """
        Fixture to adjust the printing format when running pytest with the "-s" flag - by default print messages mix up with pytest's output
        """

        print()

    @pytest.fixture(autouse=True)
    def app_config(self):
        """
        A fixture to implement setUp/tearDown functionality for all tests by initializing configuration structure for the web service
        """

        self.config = testing.setUp()
        self.config.add_settings({"influx_host": "localhost", "influx_port": 8086, "clmc_service_port": 9080})

        yield

        testing.tearDown()

    @patch('clmcservice.managementapi.views.delete')
    @patch('clmcservice.managementapi.views.InfluxDBClient')
    def test_delete_all_sfc_data(self, influx_client_mock, http_delete_mock):
        """
        Tests the functionality of the 'delete all SFC data' API endpoint.
        """

        app_settings = self.config.get_settings()

        # mock the responses of the two API calls
        mock_response1 = Mock()
        mock_response1.status_code = 200
        http_delete_mock.return_value = mock_response1

        mock_response2 = Mock()
        influx_client_mock.return_value = mock_response2

        # send a request and check that the correct API calls are made
        request = testing.DummyRequest()
        sfc_id = "test-sfc"
        request.matchdict["sfc_id"] = sfc_id

        DataManagementAPI(request).delete_all_sfc_data()

        influx_client_mock.assert_called_with(host=app_settings["influx_host"], port=app_settings["influx_port"])
        http_delete_mock.assert_called_with("http://localhost:{0}/alerts/{1}".format(app_settings["clmc_service_port"], sfc_id))
        mock_response2.drop_database.assert_called_with(sfc_id)
