#!/usr/bin/python3
"""
## © University of Southampton IT Innovation Centre, 2018
##
## Copyright in this software belongs to University of Southampton
## IT Innovation Centre of Gamma House, Enterprise Road,
## Chilworth Science Park, Southampton, SO16 7NS, UK.
##
## This software may not be used, sold, licensed, transferred, copied
## or reproduced in whole or in part in any manner or form or in or
## on any media by any person other than in accordance with the terms
## of the Licence Agreement supplied with the software, or otherwise
## without the prior written consent of the copyright owners.
##
## This software is distributed WITHOUT ANY WARRANTY, without even the
## implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
## PURPOSE, except where stated in the Licence Agreement supplied with
## the software.
##
##      Created By :            Nikolay Stanchev
##      Created Date :          22-08-2018
##      Created for Project :   FLAME
"""


from http.server import HTTPServer, BaseHTTPRequestHandler
from json import loads, dump
from os.path import join


LOG_TEST_FOLDER_PATH = "/var/log/flame/clmc/alerts"


class CustomHTTPHandler(BaseHTTPRequestHandler):
    """
    An http handler used in the integration test of CLMC alerts.
    """

    def _set_headers(self):
        """
        Sets up headers used to send back a response to a received request
        """

        self.send_response(200)
        self.send_header('Content-type', 'application/json')
        self.end_headers()

    def do_POST(self):
        """
        This method handles any POST requests made to the server - used to handle the incoming POST requests from Kapacitor.
        """

        global LOG_TEST_FOLDER_PATH

        # read post data and message ID - the topic from Kapacitor
        content_length = int(self.headers['Content-Length'])
        post_data = self.rfile.read(content_length)
        post_data = post_data.decode(self.headers.get('Accept-Charset', "utf-8"))
        post_data = loads(post_data)  # load to json
        msg_id = post_data["id"]

        # write data to log file named after the message ID
        with open(join(LOG_TEST_FOLDER_PATH, "alert-{0}.log".format(msg_id)), "w+") as fh:
            dump(fp=fh, obj=post_data)

        # send back a response (needed to mimic the behaviour of an actual http server)
        self._set_headers()
        self.wfile.write(b"{\"msg\": \"accepted\"}")


def run(server_class=HTTPServer, handler_class=BaseHTTPRequestHandler):
    """
    Starts the server on port 9999

    :param server_class: defaults to HTTPServer (standard lib)
    :param handler_class: defaults to the Base HTTP request handler (standard lib)
    """

    server_address = ('', 9999)
    httpd = server_class(server_address, handler_class)
    httpd.serve_forever()


if __name__ == "__main__":
    run(handler_class=CustomHTTPHandler)  # run server with the custom http handler
